/**
 * 作者：dailc 
 * 时间：2016-01-20 14:10:25
 * 描述： 本地缓存下载文件 
 */
(function() {
	var DownloadUtil = require('scripts/Core/DownLoadUtil.js');

	Zepto('#clearAllFileCache').on('tap', function() {
		DownloadUtil.clearAllLocalFileCache(function(msg) {
			console.log('清除缓存成功:' + msg);
		}, function(e) {
			console.log('清除缓存失败:' + e);
		});
		//这里为了测试效果,将所有的图片重置
		var imgs = document.querySelectorAll('img');
		Zepto.each(imgs, function(key, value) {
			this.setAttribute('src', '../../img/img_test.jpg');
		});
	});
	Zepto('#clearFirstFileCache').on('tap', function() {
		var imgs = document.querySelectorAll('.firstImg');
		Zepto.each(imgs, function(key, value) {
			var src = this.getAttribute('data-img-localcache');
			DownloadUtil.clearNetUrlFileCache(src);
			this.setAttribute('src', '../../img/img_test.jpg');
		});

	});
	Zepto('#downloadFirstFileWithCache').on('tap', function() {
		var imgs = document.querySelectorAll('.firstImg');
		Zepto.each(imgs, function(key, value) {
			var $this = this;
			var src = $this.getAttribute('data-img-localcache');
			downLoadImg(src,$this);
		});

	});
	Zepto('#downloadAllFileWithCache').on('tap', function() {
		//获取页面中所有的图片
		var imgs = document.querySelectorAll('img');
		Zepto.each(imgs, function(key, value) {
			var $this = this;
			var src = $this.getAttribute('data-img-localcache');
			downLoadImg(src,$this);
		});

	});

	function downLoadImg(src, dom) {
		DownloadUtil.downloadFileWidthLocalCache(src, {
			beforeDownload: function() {
				dom.setAttribute('src', '../../img/mobileFrame/img_loading.jpg');
			},
			successDownload: function(relativePath) {
				console.log('下载成功:' + relativePath);
				mui.plusReady(function() {
					var sd_path = plus.io.convertLocalFileSystemURL(relativePath);
					dom.setAttribute('src', sd_path);
				});
			},
			errorDownload: function(msg) {
				dom.setAttribute('src', '../../img/mobileFrame/img_error.jpg');
			}
		});
	};
	mui.plusReady(function() {
		var data = [
			"https://img.alicdn.com/imgextra/i2/698360784/TB2Khq1dFXXXXccXXXXXXXXXXXX_!!698360784.jpg",
			"https://img.alicdn.com/imgextra/i4/698360784/TB21UhfdVXXXXczXXXXXXXXXXXX_!!698360784.jpg",
			"https://img.alicdn.com/imgextra/i3/698360784/TB2k4WOdFXXXXXVXpXXXXXXXXXX_!!698360784.jpg",
			"https://img.alicdn.com/imgextra/i2/698360784/TB2Khq1dFXXXXccXXXXXXXXXXXX_!!698360784.jpg",
			"https://img.alicdn.com/imgextra/i4/698360784/TB21UhfdVXXXXczXXXXXXXXXXXX_!!698360784.jpg",
			"https://img.alicdn.com/imgextra/i3/698360784/TB2k4WOdFXXXXXVXpXXXXXXXXXX_!!698360784.jpg",
			//这张图片的目的是用来检测一些服务器端的特殊头像也能正常显示
			'http://221.226.86.35/EpointMobileJK/ShowHeadImg.aspx?RowGuid=6c8364c4-79cc-42a7-bb9f-e911ff22d1ed'
		];
		appendImgsWithData(data, 'imgContent');
		//通过工具类下载文件,这里进行设置
		DownloadUtil.setOptions({
			//默认的下载缓存目录-存到应用的downloads/downloadFiles下
			'downloadStoragePath': "_downloads/imgs/",
			//本地缓存的时间戳,毫秒单位,默认为1天
			'fileCacheTimeStamp': 1000 * 60 * 60 * 24 * 1,
			//同时最多的downloader 并发下载数目,默认为3个
			'concurrentDownloadCount': 1,
			//超时请求时间
			'timeout': 3,
			//超时请求后的重试次数
			'retryInterval': 3,
			//单个下载任务最大的请求时间,防止一些机型上无法触发错误回调,单位毫秒,默认10秒
			'maxTimeSingleDownloadTaskSpend': 1000 * 10,
			//获取相对路径的函数,如果不传,则用默认的路径处理方法
			'getRelativePathFromLoadUrlCallback': function(loadUrl) {
				//获取图片后缀,如果没有获取到后缀,默认是jpg
				var imgSuffix = loadUrl.substring(loadUrl.lastIndexOf(".") + 1, loadUrl.length);
				if (
					imgSuffix.toLocaleLowerCase() != ("jpg") &&
					imgSuffix.toLocaleLowerCase() != ("jpeg") &&
					imgSuffix.toLocaleLowerCase() != ("png") &&
					imgSuffix.toLocaleLowerCase() != ("bmp") &&
					imgSuffix.toLocaleLowerCase() != ("svg") &&
					imgSuffix.toLocaleLowerCase() != ("gif")
				) {
					//如果后缀没有包含以上图片,将后缀改为jpg
					imgSuffix = 'jpg';
				}
				//更换存储方式,变为将整个路径存储下来,然后去除非法字符
				var regIllegal = /[&\|\\\*^%$#@\-:.?\/=!]/g;
				//获取图片名字
				var imgName = loadUrl.replace(regIllegal, '');
				//最终的名字
				var filename = imgName + '.' + imgSuffix;
				//console.log('loadurl:'+loadUrl+',fileName:'+filename);
				var relativePath = '_downloads/imgs/' + filename;
				return relativePath;
			}
		});
	});
	/**
	 * @description 根据传入的data生成图片
	 * @param {Array} imgData
	 * @param {String} domId 
	 */
	function appendImgsWithData(imgData, domId) {
		if (!imgData || !Array.isArray(imgData) || !domId) {
			return;
		}
		//最终输出的html
		var html = '';
		//上部分的等比例图片
		var imgs_cover = '';
		//下部分的完整图片
		var imgs_container = '';
		for (var i = 0; i < imgData.length; i++) {
			if (i % 3 == 0) {
				imgs_cover += '<li class="coverImgContent">';
				imgs_cover += '<h5>图片等比例缩放,占满宽高,取中间显示</h5>';
			}
			var classExtraStr = '';
			if (i == 0) {
				classExtraStr = 'firstImg';
			}
			imgs_cover += '<img class="img_cover ' + classExtraStr + '" src="../../img/img_test.jpg" data-img-localcache="' + imgData[i] + '" />';
			if (i % 3 == 2 || i == imgData.length - 1) {
				imgs_cover += '</li>';
			}
			if (i == 0) {
				imgs_container += '<li class="containImgContent">';
				imgs_container += '<h5>等比例缩放,宽高至少占满一边,显示全部</h5>';
			}
			imgs_container += '<img class="img_contain ' + classExtraStr + '" src="../../img/img_test.jpg" data-img-localcache="' + imgData[i] + '" />';
			if (i == imgData.length - 1) {
				imgs_container += '</li>';
			}
		}
		html = imgs_cover + imgs_container;
		console.log(html);
		document.getElementById(domId).innerHTML = html;
	};
})();